#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include "gequel.h"

#include "interface.h"
#include "support.h"

/* Routine to copy all necessary field data complete with mallocs etc. */
SQL_FIELD *copyFields(SQL_FIELD *src, int numflds)
{
   int y;
   SQL_FIELD *tgt;
   
   ldebug(2,"copyFields called\n");
   
   if(src == NULL)
      return NULL;
   
   /* Malloc space and copy all normal fields */
   tgt = (SQL_FIELD *)g_malloc(sizeof(SQL_FIELD) * numflds);
   memcpy(tgt, src, sizeof(SQL_FIELD) * numflds); 

   for(y=0; y<numflds; y++)
   {
      int tlen;
      /* Copy pointer "name" as well. Note that we do not copy other strings 
       like "table" and "def" here, so fieldinfo is incomplete ! */
      
      ldebug(4,"copyFields: now copying %d : %s\n",y, src[y].name);
      
      tlen        = strlen(src[y].name) + 1;
      tgt[y].name = (char *)g_malloc(tlen);
      strncpy(tgt[y].name, src[y].name, tlen);
   }
   
   return tgt;
}

/* Routine to free all mallocated memory space in copyFields above */
void freeFields(SQL_FIELD *tgt, int numflds)
{
   int y;
   
   ldebug(2,"freeFields called\n");
   
   if(tgt == NULL)
     return;
   
   for(y=0; y<numflds; y++)
     g_free(tgt[y].name);

   g_free(tgt);
}

/* Copy relevant information for Tab to a separate structure, so there
 can be more of them all with their own private info. Note that there is no
 need to create the struct itself as all of them are already defined in
 the Global Params (we know the number of tabs at startuptime !) */
TabInfo *newTabInfostruct(Params *p, int tabid, int nrcols, int nrrows)
{
   int tlen;
   TabInfo *ti;
   GtkWidget *top = p->mainwindow;
      
   ldebug(2,"newTabInfostruct entered\n");
   
   ti = (TabInfo *)g_malloc(sizeof(TabInfo));
   if(ti == NULL)
     return NULL;
   
   ti->numflds    = p->numflds;
   ti->dbsock     = p->dbsock;
   ti->sortkey    = p->sortkey;

   memcpy(ti->styles, p->styles, sizeof(p->styles));

   ti->table = NULL;
   if(p->table != NULL)
   {
      tlen = strlen(p->table) + 1;
      ti->table = (char *)g_malloc(tlen);
      strncpy(ti->table,p->table,tlen);
   }
   
   ti->selectedrow  = -1;
   ti->selectedname = NULL;

   ti->nrcols     = nrcols; 
   ti->nrrows     = nrrows;
   ti->mainwindow = p->mainwindow;

   /* Copy complete field array from TabInfo : */
   ti->fields = copyFields(p->fields, p->numflds);
   /* Note that nrcols is not always the same as nrflds (eg in showFields) */
   ti->titles = (gchar **)g_malloc(sizeof(gchar *) * ti->nrcols);

   ti->paramshook = (void *)p; /* only to set sortKey: better solution ? */

   switch(tabid)
   {
    case RECORDSTAB :
      ti->frame = GTK_WIDGET(lookup_widget(top, "recordstab"));  break;
    case FIELDSTAB :
      ti->frame = GTK_WIDGET(lookup_widget(top, "fieldstab"));   break;
    case SEARCHTAB :
      ti->frame = GTK_WIDGET(lookup_widget(top, "searchtab"));   break;
    case QUERYTAB :
      ti->frame = GTK_WIDGET(lookup_widget(top, "querytab"));    break;
   }
   
   return ti;
}

void destroyTabInfostruct(TabInfo *ti)
{
   ldebug(2,"destroyTabInfostruct entered\n");
   if(ti == NULL)
     return;
   
   freeFields(ti->fields,ti->numflds);

   g_free(ti->titles);
   
   if(ti->table != NULL)
     g_free(ti->table);

   g_free(ti);
}


/* Stores all information in a record, so the recordwindow won't fail or crash
 * in case the main window has switched to a different table. Different
 * database is too complicated : in that case we should even consider to 
 * destroy all open recordwindows. (but what if table is changed back ?
 * database will issue errors or warnings anyway !) */
Record *newRecordstruct(TabInfo *ti)
{
   int y;
   int tlen;
   Record *rec;
   gchar *text;
   
   ldebug(2,"newRecordstruct entered\n");
   
   if(ti->table == NULL)
     return NULL;
   rec = (Record *)g_malloc(sizeof(Record));
   if(rec == NULL)
     return NULL;

   rec->numflds    = ti->numflds;
   rec->dbsock     = ti->dbsock;
   rec->sortkey    = ti->sortkey;

   memcpy(rec->styles,ti->styles,sizeof(ti->styles));

   tlen = strlen(ti->table) + 1;
   rec->table = (char *)g_malloc(tlen);
   strncpy(rec->table,ti->table,tlen);

   /* allocate content buffers for later use : */
   rec->contents = (char **)g_malloc(sizeof(char *) * ti->numflds);
   
   /* Copy complete field array from TabInfo : */
   rec->fields=copyFields(ti->fields,ti->numflds);

   /* Copy contents to record structure (for Modify, Copy & Restore): */
   for(y=0;y<ti->numflds;y++)
   {
      tlen = ti->fields[y].length + 1;
      rec->contents[y] = (char *)g_malloc(tlen);

      if(ti->selectedrow >= 0)
      {
	 gtk_clist_get_text(GTK_CLIST(ti->clist),
			    ti->selectedrow, y, &text);
	 strncpy(rec->contents[y],text,tlen);
      }
   }
   
   /* do we still need this ? yes : for reading the changed fieldcontents */
   rec->entries = (GtkEntry **)g_malloc(sizeof(GtkEntry *) * ti->numflds);

   rec->window = NULL; /* NULL until topwindow newd */
   return rec;
}

Record *copyRecordstruct(Record *src)
{
   int y;
   int tlen;
   Record *dst;
   
   ldebug(2,"copyRecordstruct entered\n");
   
   if(src == NULL)
     return NULL;
   dst = (Record *)g_malloc(sizeof(Record));
   if(dst == NULL)
     return NULL;

   dst->numflds    = src->numflds;
   dst->dbsock     = src->dbsock;
   dst->sortkey    = src->sortkey;

   memcpy(dst->styles,src->styles,sizeof(src->styles));

   tlen = strlen(src->table) + 1;
   dst->table = (char *)g_malloc(tlen);
   strncpy(dst->table,src->table,tlen);

   /* copy fieldnames and new contentbuffer for later use : */

   dst->contents = (char **)g_malloc(sizeof(char *) * src->numflds);

   /* Copy complete field array from src : */
   dst->fields = copyFields(src->fields,src->numflds);

   for(y=0;y<src->numflds;y++)
   {
      tlen = src->fields[y].length + 1;
      dst->contents[y] = (char *)g_malloc(tlen);
      /* don't copy from entries but from contents so we'll get changed 
       * fields in the source as well : */
      strncpy(dst->contents[y],gtk_entry_get_text(src->entries[y]),tlen);
   }
   
   /* do we still need this ? yes : for reading the changed fieldcontents */
   dst->entries = (GtkEntry **)g_malloc(sizeof(GtkEntry *) * src->numflds);

   return dst;
}

void destroyRecordstruct(Record *rec)
{
   int y;
   
   ldebug(2,"destroyRecordstruct entered\n");
   
   if(rec == NULL)
     return;

   freeFields(rec->fields,rec->numflds);
   
   for(y=0; y<rec->numflds; y++)
      g_free(rec->contents[y]);
   g_free(rec->contents);

   g_free(rec->table);
   g_free(rec->entries);
   
   g_free(rec);
}
